@@ -10,7 +10,7 @@ print.APML0=function(x, digits=4, ...) {
 
   if (x$family == "gaussian") {
     cat("\nRegularized linear regression: ", tem)
-  } else if (x$family == "binary") {
+  } else if (x$family == "binomial") {
     cat("\nRegularized logistic regression: ", tem)
   } else if (x$family == "cox") {
     cat("\nRegularized Cox model: ", tem)
@@ -31,14 +31,14 @@ print.APML0=function(x, digits=4, ...) {
     #print(signif(x$fit,digits))
     switch(x$family,
            "gaussian"=print(cbind(signif(x$fit[-6],digits),x$fit[6])),
-           "binary"=print(cbind(signif(x$fit[-6],digits),x$fit[6])),
+           "binomial"=print(cbind(signif(x$fit[-6],digits),x$fit[6])),
            "cox"=print(cbind(signif(x$fit[-5],digits),x$fit[5]))
     )
   } else if (any(colnames(x$fit) %in% "cvm") & !is.null(x$lambda.opt)) {
     #print(signif(x$fit,digits))
     switch(x$family,
            "gaussian"=print(cbind(signif(x$fit[-6],digits),x$fit[6])),
-           "binary"=print(cbind(signif(x$fit[-6],digits),x$fit[6])),
+           "binomial"=print(cbind(signif(x$fit[-6],digits),x$fit[6])),
            "cox"=print(cbind(signif(x$fit[-5],digits),x$fit[5]))
     )
     cat("\n\nTuning the number of non-zeros with lambda:\n\n")
